"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const common_1 = require("../../common");
const kuzzle_1 = require("../../support/kuzzle");
const http_1 = (0, tslib_1.__importDefault)(require("http"));
const ws_1 = (0, tslib_1.__importDefault)(require("ws"));
// Add the proper headers and send the body in the response
function sendResponse(response, bodyObject) {
    response.setHeader("Content-Type", "application/json; charset=UTF-8");
    response.setHeader("Cache-Control", "no-cache");
    const body = JSON.stringify(bodyObject);
    response.setHeader("Content-Length", Buffer.byteLength(body));
    response.writeHead(200);
    response.end(body);
}
class DebugProxy extends common_1.Kommand {
    async runSafe() {
        // @ts-ignore Disable ping pong since when debugging Kuzzle might not be responding
        clearInterval(this.sdk.sdk.protocol.pingIntervalId);
        const nodeVersionResponse = (await this.sdk.query({
            controller: "debug",
            action: "nodeVersion",
        }));
        this.logInfo(`Connected to Kuzzle node: ${nodeVersionResponse.node}`);
        /**
         * To allow the Chrome Debugger to see the proxy server we must implement the following routes:
         * - /json/version
         * - /json
         */
        const server = http_1.default.createServer((request, response) => {
            if (request.url === "/json/version") {
                sendResponse(response, {
                    Browser: `node.js/${nodeVersionResponse.result}`,
                    "Protocol-Version": "1.1",
                });
                return;
            }
            else if (request.url === "/json") {
                // Here we send a bunch of information about the Remote Target to the Chrome Debugger
                // Those are the informations that the Chrome Debugger uses to know how to connect to Remote Target
                // We give the Chrome Inspector our own Websocket endpoint so that it can connect to our proxy server
                sendResponse(response, [
                    {
                        description: "node.js instance",
                        devtoolsFrontendUrl: `devtools://devtools/bundled/js_app.html?experiments=true&v8only=true&ws=${request.headers.host}/kuzzle-debugger`,
                        devtoolsFrontendUrlCompat: `devtools://devtools/bundled/inspector.html?experiments=true&v8only=true&ws=${request.headers.host}/kuzzle-debugger`,
                        faviconUrl: "https://nodejs.org/static/images/favicons/favicon.ico",
                        id: "kuzzle-debugger",
                        title: `Kuzzle Debugger - ${this.flags.host}:${this.flags.port}`,
                        type: "node",
                        webSocketDebuggerUrl: `ws://${request.headers.host}/kuzzle-debugger`,
                    },
                ]);
                return;
            }
            response.writeHead(404);
        });
        const closeServer = async () => {
            this.logInfo("Connection closed.");
            if (!this.flags.noAutoEnableDebugger) {
                await this.sdk.query({
                    controller: "debug",
                    action: "disable",
                });
            }
            this.sdk.disconnect();
            server.close();
        };
        const wss = new ws_1.default.Server({
            server,
        });
        if (!this.flags.noAutoEnableDebugger) {
            await this.sdk.query({
                controller: "debug",
                action: "enable",
            });
        }
        // Listen to all events emitted by the Debug Controller
        await this.sdk.query({
            controller: "debug",
            action: "addListener",
            body: {
                event: "*",
            },
        });
        wss.on("connection", (ws) => {
            this.logInfo("Connection established.");
            /**
             * Listen to the room were events from the DebugController are sent
             * and only forward events from the Chrome  Devtools Protocol
             */
            this.sdk.sdk.protocol.on("kuzzle-debugger-event", (payload) => {
                if (!payload.event || payload.event.startsWith("Kuzzle")) {
                    return;
                }
                if (this.flags.showDebuggerEvents) {
                    this.logInfo(JSON.stringify(payload.result, null, 2));
                }
                ws.send(JSON.stringify(payload.result));
            });
            // When receiving message from Chrome Devtools Protocol, we forward it to the DebugController
            ws.on("message", async (message) => {
                try {
                    const json = JSON.parse(message.toString());
                    if (json.id === undefined || json.method === undefined) {
                        return;
                    }
                    if (this.flags.showDebuggerPayloads) {
                        this.logInfo(JSON.stringify({
                            method: json.method,
                            params: json.params,
                        }, null, 2));
                    }
                    const response = await this.sdk.query({
                        controller: "debug",
                        action: "post",
                        body: {
                            method: json.method,
                            params: json.params,
                        },
                    });
                    if (this.flags.showDebuggerPayloads) {
                        this.logInfo(JSON.stringify({
                            id: json.id,
                            result: response.result,
                        }, null, 2));
                    }
                    // Send back the response using the same ID received
                    ws.send(JSON.stringify({
                        id: json.id,
                        result: response.result,
                    }));
                }
                catch (e) {
                    this.logKo(`${e}`);
                }
            });
            ws.on("close", closeServer);
        });
        server.on("error", async (err) => {
            this.logKo(err.message);
            this.logKo(err.stack);
            await closeServer();
        });
        server.listen(this.flags.forwardPort, () => {
            this.logOk(`Listening on port ${this.flags.forwardPort}, forwarding to Kuzzle at ${this.flags.host}:${this.flags.port}`);
            this.logInfo(`Showing to Chrome Debugger as "Kuzzle Debugger - ${this.flags.host}:${this.flags.port}"`);
            this.logInfo("Waiting for Chrome Debugger to connect...");
        });
        let resolve;
        const promise = new Promise((res) => {
            resolve = res;
        });
        server.on("close", () => {
            resolve();
        });
        await promise;
    }
}
exports.default = DebugProxy;
DebugProxy.description = "Create a Proxy Server that allows Chrome to debug Kuzzle remotely using the DebugController";
DebugProxy.flags = Object.assign({ help: command_1.flags.help(), forwardPort: command_1.flags.integer({
        description: "Port of the forwarding server",
        default: 9222,
    }), ttl: command_1.flags.string({
        description: "Kuzzle login TTL",
        default: "1h",
    }), keepAuth: command_1.flags.boolean({
        description: "Keep the user authenticated",
        default: false,
    }), noAutoEnableDebugger: command_1.flags.boolean({
        description: "True if Kourou should not enable and disable the Debugger automatically before and after usage",
        default: false,
    }), showDebuggerEvents: command_1.flags.boolean({
        description: "Verbose mode to display events sent to the Chrome Debugger",
        default: false,
    }), showDebuggerPayloads: command_1.flags.boolean({
        description: "Verbose mode to display payloads sent by and to the Chrome Debugger",
        default: false,
    }) }, kuzzle_1.kuzzleFlags);
// Force the usage of Websocket otherwise each request might end up executed on a different Kuzzle Node
DebugProxy.sdkOptions = {
    protocol: "ws",
};
