"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const common_1 = require("../../common");
const kuzzle_1 = require("../../support/kuzzle");
const dump_collection_1 = require("../../support/dump-collection");
class CollectionExport extends common_1.Kommand {
    async runSafe() {
        const exportPath = this.flags.path
            ? path_1.default.join(this.flags.path, this.args.index)
            : this.args.index;
        const fields = this.flags.fields ? this.flags.fields.split(",") : [];
        if (this.flags.format === "csv" && fields.length === 0) {
            this.logInfo("It looks like you are exporting to CSV but you did not select any field. All exportable fields in the mapping will be exported.");
        }
        let query = this.parseJs(this.flags.query);
        if (this.flags.editor) {
            query = this.fromEditor(query, { json: true });
        }
        const countAll = await this.sdk.document.count(this.args.index, this.args.collection);
        const count = await this.sdk.document.count(this.args.index, this.args.collection, { query });
        this.logInfo(`Dumping collection "${this.args.index}:${this.args.collection}" in ${exportPath} ...`);
        fs_1.default.mkdirSync(exportPath, { recursive: true });
        if (this.flags.type === "all" || this.flags.type === "mappings") {
            this.logInfo(`Dumping collection mappings ...`);
            await (0, dump_collection_1.dumpCollectionMappings)(this.sdk, this.args.index, this.args.collection, exportPath, this.flags.format);
        }
        if (this.flags.type === "all" || this.flags.type === "data") {
            this.logInfo(`Dumping collection ${count} of ${countAll} documents ...`);
            await (0, dump_collection_1.dumpCollectionData)(this.sdk, this.args.index, this.args.collection, Number(this.flags["batch-size"]), exportPath, query, this.flags.format, fields, this.flags.scrollTTL);
        }
        this.logOk(`Collection ${this.args.index}:${this.args.collection} dumped`);
    }
}
exports.default = CollectionExport;
CollectionExport.keepAuth = true;
CollectionExport.description = "Exports a collection (JSONL format)";
CollectionExport.flags = Object.assign(Object.assign({ help: command_1.flags.help({}), path: command_1.flags.string({
        description: "Dump root directory",
    }), "batch-size": command_1.flags.string({
        description: "Maximum batch size (see limits.documentsFetchCount config)",
        default: "2000",
    }), query: command_1.flags.string({
        description: "Only dump documents matching the query (JS or JSON format)",
        default: "{}",
    }), editor: command_1.flags.boolean({
        description: "Open an editor (EDITOR env variable) to edit the query before sending",
    }), format: command_1.flags.string({
        description: `"kuzzle" will export in Kuzzle format usable for internal fixtures,
"jsonl" allows to import that data back with kourou,
"csv" allows to import data into Excel (please, specify the fields to export using the --fields option).`,
        options: ["jsonl", "kuzzle", "csv"],
        default: "jsonl",
    }), fields: command_1.flags.string({
        description: `[CSV format only] The list of fields to be included in the CSV export in dot-path format.

Example:
--fields oneField,anotherField,yetAnotherOne.nested.moarNested

Note that the '_id' field is always included in the CSV export. Leaving this option empty implies that all
exportable fields in the mapping will be exported.`,
    }), scrollTTL: command_1.flags.string({
        description: `The scroll TTL option to pass to the dump operation (which performs a document.search under the hood),
expressed in ms format, e.g. '2s', '1m', '3h'.`,
        default: "20s",
    }), type: command_1.flags.string({
        description: "Type of the export: all, mappings, data",
        default: "all",
    }) }, kuzzle_1.kuzzleFlags), { protocol: command_1.flags.string({
        description: "Kuzzle protocol (http or websocket)",
        default: "ws",
    }) });
CollectionExport.args = [
    { name: "index", description: "Index name", required: true },
    { name: "collection", description: "Collection name", required: true },
];
CollectionExport.examples = [
    "kourou collection:export nyc-open-data yellow-taxi",
    "kourou collection:export nyc-open-data yellow-taxi --query '{ term: { city: \"Saigon\" } }'",
];
