"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const cli_ux_1 = (0, tslib_1.__importDefault)(require("cli-ux"));
const chalk_1 = (0, tslib_1.__importDefault)(require("chalk"));
const node_emoji_1 = (0, tslib_1.__importDefault)(require("node-emoji"));
const fs_1 = require("fs");
const common_1 = require("../../common");
const providers_1 = require("../../support/migrate/providers");
class EsMigrate extends common_1.Kommand {
    constructor() {
        super(...arguments);
        this.src = undefined;
        this.dest = undefined;
    }
    async detectProviderType(provider) {
        if ((0, providers_1.isURL)(provider)) {
            return new providers_1.Elasticsearch(provider, {
                batchSize: this.flags["batch-size"],
                scrollDuration: this.flags.scroll,
            });
        }
        else if (path_1.default.basename(provider)) {
            if (!(0, providers_1.fileExists)(provider)) {
                await fs_1.promises.mkdir(provider, { recursive: true });
            }
            return new providers_1.File(provider);
        }
        else {
            throw new Error(`Unknown provider type: ${provider}. You should provide either an Elasticsearch URL or a file path.`);
        }
    }
    async migrateIndex(index) {
        const indexDefinition = await this.src.getIndex(index);
        await this.dest.createIndex(index, indexDefinition);
        return;
    }
    async migrateData(index) {
        const queue = [];
        let documents;
        const data = await this.src.readData(index);
        const scrollId = data.scrollId;
        const total = data.total;
        documents = data.documents;
        const progressBar = cli_ux_1.default.progress({
            format: chalk_1.default.blue(" [*] Importing |{bar}| {percentage}% || {value}/{total} documents"),
        });
        if (total) {
            progressBar.start(total, 0);
        }
        queue.push(documents);
        let count = 0;
        while (queue.length && total !== count) {
            documents = queue.pop();
            count += await this.dest.writeData(index, documents);
            progressBar.update(count);
            queue.push(await this.src.readData(index, { scrollId }));
        }
        progressBar.stop();
        return total;
    }
    async runMigrate(index) {
        this.logInfo(`Importing ${chalk_1.default.bold(index)} to ${chalk_1.default.bold(this.flags.dest)}`);
        await this.migrateIndex(index);
        this.logOk("Mappings successfully imported!");
        const count = await this.migrateData(index);
        if (count === 0) {
            this.logInfo("No documents to import\n");
        }
        else {
            this.logOk(`${chalk_1.default.bold(count)} document(s) imported!\n`);
        }
    }
    async runSafe() {
        this.src = await this.detectProviderType(this.flags.src);
        this.dest = await this.detectProviderType(this.flags.dest);
        if (this.flags.reset) {
            if (!this.flags["no-interactive"]) {
                this.log(chalk_1.default.red(`${node_emoji_1.default.get("fire")} Are you sure you want to reset ${chalk_1.default.bold(this.flags.dest)}?`));
                await cli_ux_1.default.confirm(chalk_1.default.redBright(` ${node_emoji_1.default.get("fire")} You will lose all the data stored in it (Type "yes" to confirm)`));
            }
            await this.dest.clear();
        }
        this.logInfo(`Fetching indices list from ${chalk_1.default.bold(this.flags.src)}`);
        const srcIndices = await this.src.listIndices(this.flags.pattern);
        if (this.flags["dry-run"]) {
            this.log(`${chalk_1.default.bold(srcIndices)}`);
            this.logInfo(`${chalk_1.default.bold(srcIndices.length)} indices will be migrated`);
            return;
        }
        this.logInfo(`Starting indices migration from ${chalk_1.default.bold(this.flags.src)} to ${chalk_1.default.bold(this.flags.dest)}`);
        for (const index of srcIndices) {
            await this.runMigrate(index);
        }
    }
}
exports.default = EsMigrate;
EsMigrate.initSdk = false;
EsMigrate.description = "Migrate all the index from an Elasticsearch (or a file) to another Elasticsearch";
EsMigrate.flags = {
    help: command_1.flags.help(),
    src: command_1.flags.string({
        description: "Migration source provider",
        env: "KUZZLE_MIGRATION_SRC",
        required: true,
    }),
    dest: command_1.flags.string({
        description: "Migration destination provider",
        env: "KUZZLE_MIGRATION_DEST",
        required: true,
    }),
    reset: command_1.flags.boolean({
        description: "Reset destination Elasticsearch server",
        default: false,
    }),
    "batch-size": command_1.flags.integer({
        description: "How many documents to move in batch per operation",
        default: 1000,
    }),
    "no-interactive": command_1.flags.boolean({
        description: "Skip confirmation interactive prompts (perfect for scripting)",
        default: false,
    }),
    "dry-run": command_1.flags.boolean({
        description: "Print witch collections will be migrated",
        default: false,
    }),
    pattern: command_1.flags.string({
        description: "Pattern to match indices to migrate",
    }),
    scroll: command_1.flags.string({
        description: "Scroll duration for Elasticsearch scrolling",
        default: "30s",
    }),
};
EsMigrate.examples = [
    "kourou es:migrate --src http://elasticsearch:9200 --dest ./my-backup --batch-size 2000 --pattern '&myindexes.collection-*'",
    "kourou es:migrate --src ./my-backup --dest http://elasticsearch:9200 --reset --batch-size 2000 --no-interactive",
];
