"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileDecrypt = void 0;
const tslib_1 = require("tslib");
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const lodash_1 = (0, tslib_1.__importDefault)(require("lodash"));
const command_1 = require("@oclif/command");
const kuzzle_vault_1 = require("kuzzle-vault");
const common_1 = require("../../common");
class FileDecrypt extends common_1.Kommand {
    async runSafe() {
        if (lodash_1.default.isEmpty(this.flags["vault-key"])) {
            throw new Error("A vault key must be provided");
        }
        if (lodash_1.default.isEmpty(this.args.file)) {
            throw new Error("An encrypted file must be provided");
        }
        let outputFile = `${this.args.file.replace(".enc", "")}`;
        if (this.flags["output-file"]) {
            outputFile = this.flags["output-file"];
        }
        if (fs_1.default.existsSync(outputFile) && !this.flags.force) {
            throw new Error(`Output file "${outputFile}" already exists. Use -f flag to overwrite it.`);
        }
        const cryptonomicon = new kuzzle_vault_1.Cryptonomicon(this.flags["vault-key"]);
        if (!fs_1.default.existsSync(this.args.file)) {
            throw new Error(`File "${this.args.file}" does not exist`);
        }
        let encryptedContent;
        try {
            encryptedContent = fs_1.default.readFileSync(this.args.file, "utf8");
        }
        catch (error) {
            throw new Error(`Cannot read encrypted content from file "${this.args.file}": ${error.message}`);
        }
        const content = cryptonomicon.decryptString(encryptedContent);
        fs_1.default.writeFileSync(outputFile, content);
        this.logOk(`Successfully decrypted content into the file ${outputFile}`);
    }
}
exports.FileDecrypt = FileDecrypt;
FileDecrypt.initSdk = false;
FileDecrypt.description = "Decrypts an encrypted file.";
FileDecrypt.examples = [
    "kourou file:decrypt books/cryptonomicon.txt.enc --vault-key <vault-key>",
    "kourou file:decrypt books/cryptonomicon.txt.enc -o books/cryptonomicon.txt --vault-key <vault-key>",
];
FileDecrypt.flags = {
    force: command_1.flags.boolean({
        char: "f",
        description: "Overwrite the output file if it already exists",
    }),
    "output-file": command_1.flags.string({
        char: "o",
        description: 'Output file (default: remove ".enc")',
    }),
    "vault-key": command_1.flags.string({
        description: "Kuzzle Vault Key (or KUZZLE_VAULT_KEY)",
        default: process.env.KUZZLE_VAULT_KEY,
    }),
};
FileDecrypt.args = [
    { name: "file", description: "Encrypted file", required: true },
];
