"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const common_1 = require("../common");
const kuzzle_1 = require("../support/kuzzle");
const restore_collection_1 = require("../support/restore-collection");
const restore_securities_1 = require("../support/restore-securities");
class Import extends common_1.Kommand {
    async runSafe() {
        const files = await this.walkDirectories(this.args.path);
        for (const file of files.mappings) {
            try {
                this.logInfo(`[collection] Start importing mappings in ${file}`);
                const dump = JSON.parse(fs_1.default.readFileSync(file, "utf8"));
                const { index, collection } = await (0, restore_collection_1.restoreCollectionMappings)(this.sdk, dump);
                this.logOk(`[collection] Imported mappings for "${index}":"${collection}"`);
            }
            catch (error) {
                this.logKo(`Error during import of ${file}: ${error.message}. Skipped.`);
            }
        }
        for (const file of files.documents) {
            try {
                this.logInfo(`[collection] Start importing documents in ${file}`);
                const { total, index, collection } = await (0, restore_collection_1.restoreCollectionData)(this.sdk, this.log.bind(this), Number(this.flags["batch-size"]), file);
                this.logOk(`[collection] Imported ${total} documents in "${index}":"${collection}"`);
            }
            catch (error) {
                this.logKo(`Error during import of ${file}: ${error.message}. Skipped.`);
            }
        }
        for (const file of files.roles) {
            try {
                this.logInfo(`[roles] Start importing roles in ${file}`);
                const dump = JSON.parse(fs_1.default.readFileSync(file, "utf8"));
                const total = await (0, restore_securities_1.restoreRoles)(this, dump, this.flags["preserve-anonymous"]);
                this.logOk(`[roles] Imported ${total} roles`);
            }
            catch (error) {
                this.logKo(`Error during import of ${file}: ${error.message}. Skipped.`);
            }
        }
        for (const file of files.profiles) {
            try {
                this.logInfo(`[profiles] Start importing profiles in ${file}`);
                const dump = JSON.parse(fs_1.default.readFileSync(file, "utf8"));
                const total = await (0, restore_securities_1.restoreProfiles)(this, dump);
                this.logOk(`[profiles] Imported ${total} profiles`);
            }
            catch (error) {
                this.logKo(`Error during import of ${file}: ${error.message}. Skipped.`);
            }
        }
        for (const file of files.usersMappings) {
            try {
                this.logInfo(`[users] Start importing users collection mappings in ${file}`);
                const dump = JSON.parse(fs_1.default.readFileSync(file, "utf8"));
                const mapping = dump.content.mapping;
                await this.sdk.security.updateUserMapping({ properties: mapping });
                this.logOk("[users] collection mappings imported");
            }
            catch (error) {
                this.logKo(`Error during import of ${file}: ${error.message}. Skipped.`);
            }
        }
        for (const file of files.users) {
            try {
                this.logInfo(`[users] Start importing users in ${file}`);
                const dump = JSON.parse(fs_1.default.readFileSync(file, "utf8"));
                const total = await (0, restore_securities_1.restoreUsers)(this, dump);
                this.logOk(`[users] Imported ${total} users`);
            }
            catch (error) {
                this.logKo(`Error during import of ${file}: ${error.message}. Skipped.`);
            }
        }
    }
    async walkDirectories(directory) {
        const entries = fs_1.default.readdirSync(directory);
        const directories = entries
            .map((entry) => path_1.default.join(directory, entry))
            .filter((dirName) => fs_1.default.lstatSync(dirName).isDirectory());
        const files = entries
            .map((entry) => path_1.default.join(directory, entry))
            .filter((fileName) => fs_1.default.lstatSync(fileName).isFile())
            .reduce((memo, file) => {
            const fileType = this.getFileType(file);
            memo[fileType].push(file);
            return memo;
        }, {
            documents: [],
            mappings: [],
            roles: [],
            profiles: [],
            usersMappings: [],
            users: [],
        });
        for (const dir of directories) {
            const { documents, mappings, roles, profiles, usersMappings, users } = await this.walkDirectories(dir);
            files.documents = files.documents.concat(documents);
            files.mappings = files.mappings.concat(mappings);
            files.roles = files.roles.concat(roles);
            files.profiles = files.profiles.concat(profiles);
            files.usersMappings = files.usersMappings.concat(usersMappings);
            files.users = files.users.concat(users);
        }
        return files;
    }
    getFileType(file) {
        if (file.endsWith(".jsonl")) {
            return "documents";
        }
        try {
            const dump = JSON.parse(fs_1.default.readFileSync(file, "utf8"));
            return dump.type;
        }
        catch (error) {
            this.logKo(`Invalid JSON file "${file}". Import skipped. ${error.message}`);
        }
    }
}
exports.default = Import;
Import.keepAuth = true;
Import.description = "Recursively imports dump files from a root directory";
Import.flags = Object.assign(Object.assign({ "preserve-anonymous": command_1.flags.boolean({
        description: "Preserve anonymous rights",
        default: false,
    }), help: command_1.flags.help({}), "batch-size": command_1.flags.string({
        description: "Maximum batch size (see limits.documentsWriteCount config)",
        default: "200",
    }) }, kuzzle_1.kuzzleFlags), { protocol: command_1.flags.string({
        description: "Kuzzle protocol (http or websocket)",
        default: "ws",
    }) });
Import.args = [
    {
        name: "path",
        description: "Root directory containing dumps",
        required: true,
    },
];
