"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const fs_1 = require("fs");
const net_1 = (0, tslib_1.__importDefault)(require("net"));
const command_1 = require("@oclif/command");
const chalk_1 = (0, tslib_1.__importDefault)(require("chalk"));
const child_process_1 = require("child_process");
const cli_ux_1 = (0, tslib_1.__importDefault)(require("cli-ux"));
const node_emoji_1 = (0, tslib_1.__importDefault)(require("node-emoji"));
const common_1 = require("../../common");
const execute_1 = require("../../support/execute");
const checkPrerequisites_1 = require("../../support/docker/checkPrerequisites");
const MIN_MAX_MAP_COUNT = 262144;
const MIN_DOCO_VERSION = "2.0.0";
const kuzzleStackV1 = (increment) => `
version: '3'

services:
  kuzzle:
    image: kuzzleio/kuzzle:1
    ports:
      - "${7512 + increment}:7512"
      - "${1883 + increment}:1883"
      - "${9229 + increment}:9229"
    cap_add:
      - SYS_PTRACE
    depends_on:
      - redis
      - elasticsearch
    environment:
      - kuzzle_services__db__client__host=http://elasticsearch:9200
      - kuzzle_services__internalCache__node__host=redis
      - kuzzle_services__memoryStorage__node__host=redis
      - NODE_ENV=development
      - DEBUG=kuzzle:*,-kuzzle:entry-point:protocols:websocket

  redis:
    image: redis:5

  elasticsearch:
    image: kuzzleio/elasticsearch:5.6.10
    ports:
      - "${9200 + increment}:9200"
    ulimits:
      nofile: 65536
    environment:
      - cluster.name=kuzzle
      - "ES_JAVA_OPTS=-Xms1024m -Xmx1024m"
`;
const kuzzleStackV2 = (increment) => `
version: '3'

services:
  kuzzle:
    image: kuzzleio/kuzzle:2
    ports:
      - "${7512 + increment}:7512"
      - "${1883 + increment}:1883"
      - "${9229 + increment}:9229"
    cap_add:
      - SYS_PTRACE
    depends_on:
      - redis
      - elasticsearch
    environment:
    - kuzzle_services__storageEngine__client__node=http://elasticsearch:9200
    - kuzzle_services__internalCache__node__host=redis
    - kuzzle_services__memoryStorage__node__host=redis
    - kuzzle_server__protocols__mqtt__enabled=true
    - kuzzle_server__protocols__mqtt__developmentMode=false
    - kuzzle_limits__loginsPerSecond=50
    - NODE_ENV=development
    - DEBUG=kuzzle:*,-kuzzle:entry-point:protocols:websocket

  redis:
    image: redis:5

  elasticsearch:
    image: kuzzleio/elasticsearch:7
    ports:
      - "${9200 + increment}:9200"
    ulimits:
      nofile: 65536
`;
class InstanceSpawn extends common_1.Kommand {
    async runSafe() {
        const portIndex = await this.findAvailablePort();
        const docoFilename = path_1.default.join(this.kourouDir, `kuzzle-stack-${portIndex}.yml`);
        const successfullCheck = this.flags.check
            ? await (0, checkPrerequisites_1.checkPrerequisites)(this)
            : true;
        if (this.flags.check && successfullCheck) {
            this.log(`\n${node_emoji_1.default.get("ok_hand")} Prerequisites are ${chalk_1.default.green.bold("OK")}!`);
        }
        else if (this.flags.check && !successfullCheck) {
            throw new Error(`${node_emoji_1.default.get("shrug")} Your system doesn't satisfy all the prerequisites. Cannot run Kuzzle.`);
        }
        this.log(chalk_1.default.grey(`\nWriting docker compose file to ${docoFilename}...`));
        (0, fs_1.writeFileSync)(docoFilename, this.generateDocoFile(this.flags.version, portIndex));
        // clean up
        await (0, execute_1.execute)("docker", "compose", "-f", docoFilename, "-p", `stack-${portIndex}`, "down");
        const doco = (0, child_process_1.spawn)("docker", [
            "compose",
            "-f",
            docoFilename,
            "-p",
            `stack-${portIndex}`,
            "up",
            "-d",
        ]);
        cli_ux_1.default.action.start(` ${node_emoji_1.default.get("rocket")} Kuzzle version ${this.flags.version} is launching`, undefined, {
            stdout: true,
        });
        doco.on("close", (docoCode) => {
            if (docoCode === 0) {
                cli_ux_1.default.action.stop("done");
                this.log(`\n${node_emoji_1.default.get("thumbsup")} ${chalk_1.default.bold("Kuzzle is booting")} in the background right now.`);
                this.log(chalk_1.default.grey("To watch the logs, run"));
                this.log(chalk_1.default.grey(`  docker compose -f ${docoFilename} -p stack-${portIndex} logs -f\n`));
                this.log(`  Kuzzle port: ${7512 + portIndex}`);
                this.log(`  MQTT port: ${1883 + portIndex}`);
                this.log(`  Node.js debugger port: ${9229 + portIndex}`);
                this.log(`  Elasticsearch port: ${9200 + portIndex}`);
            }
            else {
                cli_ux_1.default.action.stop(chalk_1.default.red(` Something went wrong: docker compose exited with ${docoCode}`));
                this.log(chalk_1.default.grey("If you want to investigate the problem, try running"));
                this.log(chalk_1.default.grey(`  docker compose -f ${docoFilename} -p stack-${portIndex} up\n`));
                throw new Error("docker compose exited with a non-zero status");
            }
        });
    }
    generateDocoFile(kuzzleMajor, portIndex) {
        if (kuzzleMajor === "1") {
            return kuzzleStackV1(portIndex);
        }
        return kuzzleStackV2(portIndex);
    }
    isPortAvailable(port) {
        return new Promise((resolve, reject) => {
            const tester = net_1.default
                .createServer()
                .once("error", (error) => {
                if (!error.message.match(/EADDRINUSE/)) {
                    reject(error);
                }
                resolve(false);
            })
                .once("listening", () => {
                tester.once("close", () => resolve(true)).close();
            })
                .listen(port);
        });
    }
    async findAvailablePort() {
        let i = 0;
        // eslint-disable-next-line
        while (true) {
            if (await this.isPortAvailable(7512 + i)) {
                return i;
            }
            i++;
        }
    }
}
exports.default = InstanceSpawn;
InstanceSpawn.initSdk = false;
InstanceSpawn.description = "Spawn a new Kuzzle instance";
InstanceSpawn.flags = {
    help: command_1.flags.help(),
    check: command_1.flags.boolean({
        description: "Check prerequisite before running Kuzzle",
        default: false,
    }),
    version: command_1.flags.string({
        char: "v",
        description: "Core-version of the instance to spawn",
        default: "2",
    }),
};
