"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const promises_1 = (0, tslib_1.__importDefault)(require("node:fs/promises"));
const ndjson_1 = (0, tslib_1.__importDefault)(require("ndjson"));
const command_1 = require("@oclif/command");
const PaasKommand_1 = require("../../../support/PaasKommand");
class PaasEsDump extends PaasKommand_1.PaasKommand {
    async runSafe() {
        // Check that the batch size is positive
        if (this.flags["batch-size"] <= 0) {
            this.logKo(`The batch size must be greater than zero. (Specified batch size: ${this.flags["batch-size"]})`);
            process.exit(1);
        }
        // Log in to the PaaS
        const apiKey = await this.getCredentials();
        await this.initPaasClient({ apiKey });
        const user = await this.paas.auth.getCurrentUser();
        this.logInfo(`Logged as "${user._id}" for project "${this.flags.project || this.getProject()}"`);
        // Create the dump directory
        await promises_1.default.mkdir(this.args.dumpDirectory, { recursive: true });
        // Dump the indexes
        this.logInfo("Dumping Elasticsearch indexes...");
        const indexesResult = await this.getAllIndexes();
        await promises_1.default.writeFile(path_1.default.join(this.args.dumpDirectory, "indexes.json"), JSON.stringify(indexesResult));
        this.logOk("Elasticsearch indexes dumped!");
        // Dump all the documents
        this.logInfo("Dumping Elasticsearch documents...");
        await this.dumpAllDocuments();
        this.logOk("Elasticsearch documents dumped!");
        this.logOk(`The dumped files are available under "${path_1.default.resolve(this.args.dumpDirectory)}"`);
    }
    /**
      * @description Get all indexes from the Elasticsearch of the PaaS application.
      * @returns The indexes.
      */
    async getAllIndexes() {
        const { result } = await this.paas.query({
            controller: "application/storage",
            action: "getIndexes",
            environmentId: this.args.environment,
            projectId: this.flags.project || this.getProject(),
            applicationId: this.args.applicationId,
            body: {},
        });
        return result;
    }
    /**
      * @description Dump documents from the Elasticsearch of the PaaS application.
      * @param pitId ID of the PIT opened on Elasticsearch.
      * @param searchAfter Cursor for dumping documents after a certain one.
      * @returns The dumped documents.
      */
    async dumpDocuments(pitId, searchAfter) {
        const { result } = await this.paas.query({
            controller: "application/storage",
            action: "dumpDocuments",
            environmentId: this.args.environment,
            projectId: this.flags.project || this.getProject(),
            applicationId: this.args.applicationId,
            body: {
                pitId,
                searchAfter: JSON.stringify(searchAfter),
                size: this.flags["batch-size"],
            },
        });
        return result;
    }
    async dumpAllDocuments() {
        // Prepare dumping all documents
        let pitId = "";
        let searchAfter = [];
        let dumpedDocuments = 0;
        let totalDocuments = 0;
        const fd = await promises_1.default.open(path_1.default.join(this.args.dumpDirectory, "documents.jsonl"), "w");
        const writeStream = fd.createWriteStream();
        const ndjsonStream = ndjson_1.default.stringify();
        writeStream.on("error", (error) => {
            throw error;
        });
        ndjsonStream.on("data", (line) => {
            writeStream.write(line);
        });
        const teardown = async () => {
            // Finish the dump session if a PIT ID is set
            if (pitId.length > 0) {
                await this.finishDump(pitId);
            }
            // Close the open streams/file
            writeStream.close();
            await fd.close();
        };
        try {
            // Dump the first batch
            let result = await this.dumpDocuments(pitId, searchAfter);
            let hits = result.hits.hits;
            while (hits.length > 0) {
                // Update the PIT ID and the cursor for the next dump
                pitId = result.pit_id;
                searchAfter = hits[hits.length - 1].sort;
                // Save the documents
                for (let i = 0; i < hits.length; ++i) {
                    ndjsonStream.write(hits[i]);
                }
                dumpedDocuments += hits.length;
                totalDocuments = result.hits.total.value;
                this.logInfo(`Dumping Elasticsearch documents: ${Math.floor(dumpedDocuments / totalDocuments * 100)}% (${dumpedDocuments}/${totalDocuments})`);
                // Dump the next batch
                result = await this.dumpDocuments(pitId, searchAfter);
                hits = result.hits.hits;
            }
        }
        catch (error) {
            teardown();
            this.logKo(`Error while dumping the documents: ${error}`);
            process.exit(1);
        }
        // Finish the dump
        teardown();
    }
    /**
      * @description Finish the document dumping session.
      * @param pitId ID of the PIT opened on Elasticsearch.
      */
    async finishDump(pitId) {
        try {
            await this.paas.query({
                controller: "application/storage",
                action: "finishDumpDocuments",
                environmentId: this.args.environment,
                projectId: this.flags.project || this.getProject(),
                applicationId: this.args.applicationId,
                body: {
                    pitId,
                },
            });
        }
        catch (error) {
            this.logInfo(`Unable to cleanly finish the dump session: ${error}`);
        }
    }
}
PaasEsDump.description = "Dump data from the Elasticsearch of a PaaS application";
PaasEsDump.flags = {
    help: command_1.flags.help(),
    project: command_1.flags.string({
        description: "Current PaaS project",
    }),
    "batch-size": command_1.flags.integer({
        description: "Maximum batch size",
        default: 2000,
    }),
};
PaasEsDump.args = [
    {
        name: "environment",
        description: "Project environment name",
        required: true,
    },
    {
        name: "applicationId",
        description: "Application Identifier",
        required: true,
    },
    {
        name: "dumpDirectory",
        description: "Directory where to store dump files",
        required: true,
    }
];
exports.default = PaasEsDump;
