"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const readline = (0, tslib_1.__importStar)(require("readline"));
const command_1 = require("@oclif/command");
const chalk_1 = (0, tslib_1.__importDefault)(require("chalk"));
const chrono = (0, tslib_1.__importStar)(require("chrono-node"));
const PaasKommand_1 = require("../../support/PaasKommand");
class PaasLogs extends PaasKommand_1.PaasKommand {
    constructor() {
        super(...arguments);
        /**
         * Allowed colors for pod names.
         */
        this.allColors = [
            chalk_1.default.red,
            chalk_1.default.green,
            chalk_1.default.yellow,
            chalk_1.default.blue,
            chalk_1.default.magenta,
            chalk_1.default.cyan,
            chalk_1.default.gray,
        ];
        /**
         * Available colors for pod names.
         */
        this.availableColors = [...this.allColors];
        /**
         * The color to use for pod names.
         * @private
         */
        this.podsColor = new Map();
    }
    async runSafe() {
        var e_1, _a;
        const apiKey = await this.getCredentials();
        await this.initPaasClient({ apiKey });
        const user = await this.paas.auth.getCurrentUser();
        this.logInfo(`Logged as "${user._id}" for project "${this.flags.project || this.getProject()}"`);
        const separator = "\t";
        // Parse the time arguments
        const since = this.flags.since
            ? chrono.parseDate(this.flags.since).toISOString()
            : undefined;
        const until = this.flags.until
            ? chrono.parseDate(this.flags.until).toISOString()
            : undefined;
        // Perform the streamed request
        const incomingMessage = await this.paas.queryHttpStream({
            controller: "application",
            action: "logs",
            environmentId: this.args.environment,
            projectId: this.flags.project || this.getProject(),
            applicationId: this.args.application,
            follow: this.flags.follow,
            tailLines: this.flags.tail,
            podName: this.flags.podName,
            since,
            until,
        });
        // Don't continue if an error occurred
        if (incomingMessage.statusCode !== 200) {
            return new Promise((_, reject) => {
                // Collect the whole response body
                let responseBody = "";
                incomingMessage.on("data", (buffer) => {
                    responseBody += buffer.toString();
                });
                incomingMessage.on("end", () => {
                    let response;
                    try {
                        response = JSON.parse(responseBody);
                    }
                    catch (error) {
                        reject(new Error("An error occurred while parsing the error response body from Kuzzle"));
                        return;
                    }
                    reject(response.error);
                });
            });
        }
        // Read the response line by line
        const lineStream = readline.createInterface({
            input: incomingMessage,
            crlfDelay: Infinity,
            terminal: false,
        });
        try {
            // Display the response
            for (var lineStream_1 = (0, tslib_1.__asyncValues)(lineStream), lineStream_1_1; lineStream_1_1 = await lineStream_1.next(), !lineStream_1_1.done;) {
                const line = lineStream_1_1.value;
                try {
                    const data = JSON.parse(line);
                    // Exclude logs that are empty or that are not from a pod
                    if (!data.content || !data.podName) {
                        continue;
                    }
                    // Get the pod color
                    const podColor = this.getPodColor(data.podName);
                    // Display the log
                    const timestamp = this.flags.timestamp
                        ? `[${new Date(data.timeStamp).toLocaleString()}] `
                        : "";
                    const name = podColor(`${data.podName}${separator}`);
                    this.log(`${timestamp}${name}| ${data.content}`);
                }
                catch (error) {
                    this.logKo(`Error while parsing log: ${error} (Received: "${line}")`);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (lineStream_1_1 && !lineStream_1_1.done && (_a = lineStream_1.return)) await _a.call(lineStream_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    /**
     * Returns the color to use for a pod name.
     * @param podName Name of the pod.
     * @returns The color to use.
     */
    getPodColor(podName) {
        // Attempt to get the color from the list
        let podColor = this.podsColor.get(podName);
        if (!podColor) {
            // Get a random color
            const index = Math.floor(Math.random() * this.availableColors.length);
            [podColor] = this.availableColors.splice(index, 1);
            this.podsColor.set(podName, podColor);
            // If all colors are used, reset the available colors
            if (this.availableColors.length === 0) {
                this.availableColors = [...this.allColors];
            }
        }
        return podColor;
    }
}
PaasLogs.description = "Show logs of the targeted application";
PaasLogs.flags = {
    follow: command_1.flags.boolean({
        char: "f",
        description: "Follow log output",
    }),
    timestamp: command_1.flags.boolean({
        char: "t",
        description: "Show timestamp",
    }),
    help: command_1.flags.help(),
    project: command_1.flags.string({
        description: "Current PaaS project",
    }),
    tail: command_1.flags.integer({
        char: "n",
        description: "Number of lines to show from the end of the logs",
    }),
    podName: command_1.flags.string({
        description: "Name of the pod to show logs from",
    }),
    since: command_1.flags.string({
        description: "Display logs from a specific absolute (e.g. 2022/12/02 09:41) or relative (e.g. a minute ago) time",
    }),
    until: command_1.flags.string({
        description: "Display logs until a specific absolute (e.g. 2022/12/02 09:41) or relative (e.g. a minute ago) time",
    }),
};
PaasLogs.args = [
    {
        name: "environment",
        description: "Kuzzle PaaS environment",
        required: true,
    },
    {
        name: "application",
        description: "Kuzzle PaaS application",
        required: true,
    },
];
exports.default = PaasLogs;
