"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const lodash_1 = require("lodash");
const editor_1 = require("../../support/editor");
const common_1 = require("../../common");
const kuzzle_1 = require("../../support/kuzzle");
const typescript_1 = (0, tslib_1.__importDefault)(require("typescript"));
const node_vm_1 = (0, tslib_1.__importDefault)(require("node:vm"));
class SdkExecute extends common_1.Kommand {
    constructor() {
        super(...arguments);
        this.code = "";
    }
    async beforeConnect() {
        this.code = this.stdin || this.args.code || "// paste your code here";
        try {
            node_vm_1.default.runInContext(this.code, node_vm_1.default.createContext({}));
        }
        catch (e) {
            if (e.name === "SyntaxError") {
                const result = typescript_1.default.transpileModule(this.code, { compilerOptions: { module: typescript_1.default.ModuleKind.CommonJS } });
                this.code = result.outputText;
            }
        }
        if (this.haveSubscription) {
            this.sdkOptions.protocol = "ws";
        }
    }
    getVariables() {
        return (this.flags.var || [])
            .map((nameValue) => {
            const [name, value] = nameValue.split("=");
            return `    let ${name} = ${value};`;
        })
            .join("\n");
    }
    async runSafe() {
        if ((0, lodash_1.isEmpty)(this.code)) {
            throw new Error("No code provided.");
        }
        let userError = null;
        this.code = `
(async () => {
  try {
${this.getVariables()}
    ${this.code}
  }
  catch (error) {
    userError = error
  }
})();
`;
        // content from user editor
        if (this.flags.editor) {
            const editor = new editor_1.Editor(this.code);
            editor.run();
            this.code = editor.content;
        }
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const sdk = this.sdk.sdk;
        let result;
        try {
            // eslint-disable-next-line no-eval
            result = await eval(this.code);
        }
        catch (error) {
            userError = error;
        }
        if (userError) {
            this.logKo(`Error when executing SDK code: ${userError}`);
            this.log(userError.stack);
        }
        else {
            this.logOk("Successfully executed SDK code");
            if (result !== undefined) {
                // eslint-disable-next-line no-console
                console.log(JSON.stringify(result, null, 2));
            }
        }
        if (!userError && (this.haveSubscription || this.flags["keep-alive"])) {
            this.logInfo("Keep alive for realtime notifications ...");
            // eslint-disable-next-line @typescript-eslint/no-empty-function
            await new Promise(() => { });
        }
    }
    get haveSubscription() {
        return this.code.includes("sdk.realtime.subscribe");
    }
}
SdkExecute.description = `
Executes arbitrary code.

Code Execution

  Provided code will be executed in an async method.
  You can access a connected and authenticated SDK with the "sdk" variable.
  Templated variable passed as the command arguments are also accessible within the same name.
  Returned value will be printed on the standard output (e.g. 'return await sdk.server.now();').
  Errors will be caught and printed on the error output (e.g. 'throw new Error("failure");').

Provide code

  code can be passed as an argument
  code will be read from STDIN if available

  Examples:
    - kourou sdk:execute 'return await sdk.server.now()'
    - kourou sdk:execute 'return await sdk.index.exists(index)' --var 'index="iot-data"'
    - kourou sdk:execute < snippet.js
    - echo 'return await sdk.server.now()' | kourou sdk:execute

Other

  use the --editor flag to modify the code before executing it

  Examples:
    - kourou sdk:execute 'return await sdk.server.now()' --editor
`;
SdkExecute.flags = Object.assign({ help: command_1.flags.help(), var: command_1.flags.string({
        char: "v",
        description: "Additional arguments injected into the code. (eg: --var 'index=\"iot-data\"'",
        multiple: true,
    }), editor: command_1.flags.boolean({
        description: "Open an editor (EDITOR env variable) to edit the code before executing it.",
    }), "keep-alive": command_1.flags.boolean({
        description: "Keep the connection running (websocket only)",
    }), "print-raw": command_1.flags.boolean({
        description: "Print only the script result to stdout",
    }) }, kuzzle_1.kuzzleFlags);
SdkExecute.args = [
    {
        name: "code",
        description: "Code to execute. Will be read from STDIN if available.",
        required: false,
    },
];
SdkExecute.readStdin = true;
exports.default = SdkExecute;
