"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const lodash_1 = (0, tslib_1.__importDefault)(require("lodash"));
const common_1 = require("../../common");
const kuzzle_1 = require("../../support/kuzzle");
class SdkQuery extends common_1.Kommand {
    async runSafe() {
        const [controller, action] = this.args["controller:action"].split(":");
        const requestArgs = {};
        requestArgs.index = this.flags.index;
        requestArgs.collection = this.flags.collection;
        requestArgs._id = this.flags.id;
        for (const keyValue of this.flags.arg || []) {
            const key = keyValue.substr(0, keyValue.indexOf("="));
            const value = keyValue.substr(keyValue.indexOf("=") + 1);
            requestArgs[key] = value;
        }
        const body = this.stdin ? this.stdin : this.flags.body;
        let request = Object.assign(Object.assign({ controller,
            action }, requestArgs), { body: this.parseJs(body) });
        // content from user editor
        if (this.flags.editor && this.flags["body-editor"]) {
            throw new Error("You cannot specify --editor and --body-editor at the same time");
        }
        else if (this.flags.editor) {
            request = this.fromEditor(request, { json: true });
        }
        else if (this.flags["body-editor"]) {
            request.body = this.fromEditor(request.body, { json: true });
        }
        const response = await this.sdk.query(request);
        const display = this.flags.display === ""
            ? response
            : lodash_1.default.get(response, this.flags.display);
        console.log(JSON.stringify(display, null, 2));
        this.logOk(`Successfully executed "${controller}:${action}"`);
    }
}
SdkQuery.description = `
Executes an API query.

Query arguments

  Arguments can be passed and repeated using the --arg or -a flag.
  Index and collection names can be passed with --index (-i) and --collection (-c) flags
  ID can be passed with the --id flag.

  Examples:
    - kourou sdk:query document:delete -i iot -c sensors -a refresh=wait_for

Query body

  Body can be passed with the --body flag with either a JSON or JS string.
  Body will be read from STDIN if available

  Examples:
    - kourou sdk:query document:create -i iot -c sensors --body '{creation: Date.now())}'
    - kourou sdk:query admin:loadMappings < mappings.json
    - echo '{dynamic: "strict"}' | kourou sdk:query collection:create -i iot -c sensors

Other

  Use the --editor flag to modify the query before sending it to Kuzzle
  Use the --display flag to display a specific property of the response

  Examples:
    - kourou sdk:query document:create -i iot -c sensors --editor
    - kourou sdk:query server:now --display 'result.now'

Default fallback to API action

  It's possible to use the "sdk:query" command by only specifying the corresponding controller
  and action as first argument.

  Kourou will try to infer the first arguments to one the following pattern:
    - <command> <index>
    - <command> <body>
    - <command> <index> <collection>
    - <command> <index> <collection> <id>
    - <command> <index> <collection> <body>
    - <command> <index> <collection> <id> <body>

  If a flag is given (-i, -c, --body or --id), then the flag value has priority over
  argument infering.

  Examples:
    - kourou collection:list iot
    - kourou security:createUser '{ "content": { "profileIds": ["default"] } }' --id yagmur
    - kourou collection:delete iot sensors
    - kourou document:createOrReplace iot sensors sigfox-1 '{}'
    - kourou bulk:import iot sensors '{ bulkData: [...] }'
    - kourou admin:loadMappings < mappings.json
`;
SdkQuery.flags = Object.assign({ help: command_1.flags.help(), arg: command_1.flags.string({
        char: "a",
        description: 'Additional argument. Repeatable. (e.g. "-a refresh=wait_for")',
        multiple: true,
    }), body: command_1.flags.string({
        description: "Request body in JS or JSON format. Will be read from STDIN if available.",
        default: "{}",
    }), editor: command_1.flags.boolean({
        description: "Open an editor (EDITOR env variable) to edit the request before sending.",
    }), "body-editor": command_1.flags.boolean({
        description: "Open an editor (EDITOR env variable) to edit the body before sending.",
    }), index: command_1.flags.string({
        char: "i",
        description: "Index argument",
    }), collection: command_1.flags.string({
        char: "c",
        description: "Collection argument",
    }), id: command_1.flags.string({
        description: "ID argument (_id)",
    }), display: command_1.flags.string({
        description: "Path of the property to display from the response (empty string to display the result)",
        default: "result",
    }), "print-raw": command_1.flags.boolean({
        description: "Print only the query result to stdout",
    }) }, kuzzle_1.kuzzleFlags);
SdkQuery.args = [
    {
        name: "controller:action",
        description: 'Controller and action (eg: "server:now")',
        required: true,
    },
];
SdkQuery.readStdin = true;
exports.default = SdkQuery;
