"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const lodash_1 = (0, tslib_1.__importDefault)(require("lodash"));
const crypto_1 = (0, tslib_1.__importDefault)(require("crypto"));
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const command_1 = require("@oclif/command");
const common_1 = require("../../common");
const kuzzle_1 = require("../../support/kuzzle");
function randomPassword() {
    return crypto_1.default.randomBytes(20).toString("hex");
}
class UserExport extends common_1.Kommand {
    constructor() {
        super(...arguments);
        this.excludes = [];
    }
    async beforeConnect() {
        if (this.flags["generated-username"] !== "_id" &&
            !this.flags["generate-credentials"]) {
            throw new Error('The "--generated-username" cannot be used without "--generate-credentials"');
        }
    }
    async runSafe() {
        const filename = path_1.default.join(this.flags.path, "users.json");
        this.logInfo(`Exporting users in ${filename} ...`);
        fs_1.default.mkdirSync(this.flags.path, { recursive: true });
        this.excludes = this.flags.exclude || [];
        const users = await this._dumpUsers();
        const dump = {
            type: "users",
            content: users,
        };
        fs_1.default.writeFileSync(filename, JSON.stringify(dump, null, 2));
        this.logOk(`${Object.keys(users).length} users dumped`);
        if (this.flags["generate-credentials"]) {
            this.logOk("Credentials have been generated");
        }
    }
    async _dumpUsers() {
        const users = {};
        let results = await this.sdk.security.searchUsers({}, { scroll: "5s", size: this.flags["batch-size"] });
        while (results) {
            for (const user of results.hits) {
                const shouldInclude = this.excludes.every((exclude) => !user._id.match(new RegExp(exclude)));
                if (!shouldInclude) {
                    continue;
                }
                users[user._id] = {
                    content: lodash_1.default.omit(user.content, ["_kuzzle_info"]),
                    credentials: {},
                };
                if (this.flags["generate-credentials"]) {
                    let username = `${lodash_1.default.get(user, this.flags["generated-username"])}`;
                    const password = randomPassword();
                    // Check for empty string or undefined property converted to string
                    if (lodash_1.default.isEmpty(username) || username === "undefined") {
                        this.logInfo(`User ${user._id} does not have a "${this.flags["generated-username"]}" property. Use user ID instead.`);
                        username = user._id;
                    }
                    users[user._id].credentials.local = { username, password };
                }
            }
            results = await results.next();
        }
        return users;
    }
}
exports.default = UserExport;
UserExport.description = `
Exports users to JSON.

The users will be exported WITHOUT their credentials since Kuzzzle can't access them.

You can either:
  - Manually re-create credentials for your users
  - Use the "mustChangePasswordIfSetByAdmin" option Kuzzle password policies (see https://github.com/kuzzleio/kuzzle-plugin-auth-passport-local/#optional-properties)
  - Use the "--generate-credentials" flag to auto-generate credentials for your users

Auto-generation of credentials

  With the "--generate-credentials" flag, Kourou will add credentials for the "local" strategy.
  By default, the username will be the user ID.
  Use the "generated-username" flag to use an other property than the user ID for the generated username
  The password will be a strong random 40 characters string

Examples:

  - kourou user:export
  - kourou user:export --exclude '.*admin.*' --exclude 'supervisor.*'
  - kourou user:export --generate-credentials
  - kourou user:export --generate-credentials --generated-username content.email
`;
UserExport.flags = Object.assign(Object.assign({ help: command_1.flags.help({}), path: command_1.flags.string({
        description: "Dump directory",
        default: "users",
    }), exclude: command_1.flags.string({
        description: "Exclude users by matching their IDs with a regexp",
        multiple: true,
    }), "generate-credentials": command_1.flags.boolean({
        description: "Generate credentials with a random password for users",
    }), "generated-username": command_1.flags.string({
        description: "User content property used as a username for local credentials",
        default: "_id",
    }), "batch-size": command_1.flags.string({
        description: "Maximum batch size (see limits.documentsFetchCount config)",
        default: "2000",
    }) }, kuzzle_1.kuzzleFlags), { protocol: command_1.flags.string({
        description: "Kuzzle protocol (http or websocket)",
        default: "ws",
    }) });
