"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VaultDecrypt = void 0;
const tslib_1 = require("tslib");
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const lodash_1 = (0, tslib_1.__importDefault)(require("lodash"));
const command_1 = require("@oclif/command");
const kuzzle_vault_1 = require("kuzzle-vault");
const common_1 = require("../../common");
class VaultDecrypt extends common_1.Kommand {
    async runSafe() {
        if (lodash_1.default.isEmpty(this.flags["vault-key"])) {
            throw new Error("A vault key must be provided");
        }
        if (lodash_1.default.isEmpty(this.args.file)) {
            throw new Error("A secrets file must be provided");
        }
        let outputFile = `${this.args.file.replace(".enc", "")}`;
        if (this.flags["output-file"]) {
            outputFile = this.flags["output-file"];
        }
        if (fs_1.default.existsSync(outputFile) && !this.flags.force) {
            throw new Error(`Output file "${outputFile}" already exists. Use -f flag to overwrite it.`);
        }
        const cryptonomicon = new kuzzle_vault_1.Cryptonomicon(this.flags["vault-key"]);
        if (!fs_1.default.existsSync(this.args.file)) {
            throw new Error(`File "${this.args.file}" does not exists`);
        }
        const PARSER = kuzzle_vault_1.Vault.getParser(this.args.file);
        let encryptedSecrets = {};
        try {
            encryptedSecrets = PARSER.parse(fs_1.default.readFileSync(this.args.file, "utf8"));
        }
        catch (error) {
            throw new Error(`Cannot read secrets from file "${this.args.file}": ${error.message}`);
        }
        const secrets = cryptonomicon.decryptObject(encryptedSecrets);
        fs_1.default.writeFileSync(outputFile, PARSER.stringify(secrets, null, 2));
        this.logOk(`Secrets were successfully decrypted into the file ${outputFile}`);
    }
}
exports.VaultDecrypt = VaultDecrypt;
VaultDecrypt.initSdk = false;
VaultDecrypt.description = `
Decrypts an entire secrets file.

Decrypted secrets file must NEVER be committed into the repository.

See https://github.com/kuzzleio/kuzzle-vault/ for more information.
`;
VaultDecrypt.examples = [
    "kourou vault:decrypt config/secrets.enc.json --vault-key <vault-key>",
    "kourou vault:decrypt config/secrets.enc.json -o config/secrets.json --vault-key <vault-key>",
];
VaultDecrypt.flags = {
    force: command_1.flags.boolean({
        char: "f",
        description: "Overwrite the output file if it already exists",
    }),
    "output-file": command_1.flags.string({
        char: "o",
        description: 'Output file (default: remove ".enc")',
    }),
    "vault-key": command_1.flags.string({
        description: "Kuzzle Vault Key (or KUZZLE_VAULT_KEY)",
        default: process.env.KUZZLE_VAULT_KEY,
    }),
};
VaultDecrypt.args = [
    {
        name: "file",
        description: "File containing encrypted secrets",
        required: true,
    },
];
