"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VaultEncrypt = void 0;
const tslib_1 = require("tslib");
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const lodash_1 = (0, tslib_1.__importDefault)(require("lodash"));
const command_1 = require("@oclif/command");
const kuzzle_vault_1 = require("kuzzle-vault");
const common_1 = require("../../common");
class VaultEncrypt extends common_1.Kommand {
    async runSafe() {
        if (lodash_1.default.isEmpty(this.flags["vault-key"])) {
            throw new Error("A vault key must be provided");
        }
        if (lodash_1.default.isEmpty(this.args.file)) {
            throw new Error("A secrets file must be provided");
        }
        const { name, ext } = path_1.default.parse(this.args.file);
        let outputFile = `${name}.enc${ext}`;
        if (this.flags["output-file"]) {
            outputFile = this.flags["output-file"];
        }
        if (fs_1.default.existsSync(outputFile) && !this.flags.force) {
            throw new Error(`Output file "${outputFile}" already exists. Use -f flag to overwrite it.`);
        }
        const cryptonomicon = new kuzzle_vault_1.Cryptonomicon(this.flags["vault-key"]);
        if (!fs_1.default.existsSync(this.args.file)) {
            throw new Error(`File "${this.args.file}" does not exists`);
        }
        const PARSER = kuzzle_vault_1.Vault.getParser(this.args.file);
        let secrets = {};
        try {
            secrets = PARSER.parse(fs_1.default.readFileSync(this.args.file, "utf8"));
        }
        catch (error) {
            throw new Error(`Cannot read secrets from file "${this.args.file}": ${error.message}`);
        }
        const encryptedSecrets = cryptonomicon.encryptObject(secrets);
        fs_1.default.writeFileSync(outputFile, PARSER.stringify(encryptedSecrets, null, 2));
        this.logOk(`Secrets were successfully encrypted into the file ${outputFile}`);
    }
}
exports.VaultEncrypt = VaultEncrypt;
VaultEncrypt.initSdk = false;
VaultEncrypt.description = `
Encrypts an entire secrets file.

The secrets file must be in JSON format and it must contain only strings or objects.

Example:
{
  aws: {
    s3: {
      keyId: 'b61e267676660c314b006b06'
    }
  }
}

Encrypted secrets are meant to be loaded inside an application with Kuzzle Vault.

See https://github.com/kuzzleio/kuzzle-vault/ for more information.
`;
VaultEncrypt.examples = [
    "kourou vault:encrypt config/secrets.json --vault-key <vault-key>",
    "kourou vault:encrypt config/secrets.json -o config/secrets_prod.enc.json --vault-key <vault-key>",
];
VaultEncrypt.flags = {
    force: command_1.flags.boolean({
        char: "f",
        description: "Overwrite the output file if it already exists",
    }),
    "output-file": command_1.flags.string({
        char: "o",
        description: "Output file (default: <file>.enc.json)",
    }),
    "vault-key": command_1.flags.string({
        description: "Kuzzle Vault Key (or KUZZLE_VAULT_KEY)",
        default: process.env.KUZZLE_VAULT_KEY,
    }),
};
VaultEncrypt.args = [
    {
        name: "file",
        description: "File containing unencrypted secrets",
        required: true,
    },
];
