"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatValueForCSV = exports.pickValues = exports.Kommand = void 0;
const tslib_1 = require("tslib");
const os_1 = (0, tslib_1.__importDefault)(require("os"));
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const command_1 = require("@oclif/command");
const chalk_1 = (0, tslib_1.__importDefault)(require("chalk"));
const node_emoji_1 = (0, tslib_1.__importDefault)(require("node-emoji"));
const get_1 = (0, tslib_1.__importDefault)(require("lodash/get"));
const isObject_1 = (0, tslib_1.__importDefault)(require("lodash/isObject"));
const kepler_companion_1 = (0, tslib_1.__importDefault)(require("kepler-companion"));
const kuzzle_1 = require("./support/kuzzle");
const editor_1 = require("./support/editor");
class Kommand extends command_1.Command {
    constructor(argv, config) {
        super(argv, config);
        // Instantiate a dummy SDK to avoid the this.sdk notation everywhere -_-
        this.sdk = new kuzzle_1.KuzzleSDK({ host: "nowhere" });
        this.exitCode = 0;
        this.telemetry = new kepler_companion_1.default();
        this.stdin = undefined;
        this.sdkOptions = {};
        this.kourouDir = path_1.default.join(os_1.default.homedir(), ".kourou");
        if (process.env.KOUROU_USAGE && process.env.KOUROU_USAGE !== "true") {
            this.telemetry.turnOff();
        }
    }
    get logSilent() {
        return this.flags["print-raw"] || this.constructor.disableLog;
    }
    createKourouDir() {
        if (!fs_1.default.existsSync(this.kourouDir)) {
            fs_1.default.mkdirSync(this.kourouDir);
        }
    }
    printCommand() {
        const klass = this.constructor;
        const shortDescription = klass.description
            .split("\n")
            .filter((line) => line.length > 0)[0];
        this.log("");
        this.log(`${chalk_1.default.blue.bold(`${node_emoji_1.default.get("rocket")} Kourou`)} - ${shortDescription}`);
        this.log("");
    }
    log(message) {
        if (this.logSilent) {
            return;
        }
        super.log(` ${message}`);
    }
    logOk(message) {
        if (this.logSilent) {
            return;
        }
        this.log(chalk_1.default.green(`[✔] ${message}`));
    }
    logInfo(message) {
        if (this.logSilent) {
            return;
        }
        this.log(chalk_1.default.yellow(`[ℹ] ${message}`));
    }
    logKo(message) {
        if (this.logSilent) {
            return;
        }
        this.exitCode = 1;
        this.log(chalk_1.default.red(`[X] ${message}`));
    }
    async run() {
        const kommand = this.constructor;
        const result = this.parse(kommand);
        this.args = result.args;
        this.flags = result.flags;
        this.printCommand();
        if (kommand.readStdin) {
            this.stdin = this.fromStdin();
            if (this.stdin && this.flags.editor) {
                throw new Error("Unable to use flag --editor when reading from STDIN");
            }
        }
        // Lifecycle hook
        await this.beforeConnect();
        let err;
        try {
            if (kommand.initSdk) {
                this.sdk = new kuzzle_1.KuzzleSDK(Object.assign(Object.assign(Object.assign({}, this.flags), this.sdkOptions), { appName: this.config.name, appVersion: this.config.version, keepAuth: kommand.keepAuth }));
                await this.sdk.init(this);
            }
            if (this.flags.as) {
                this.logInfo(`Impersonate user "${this.flags.as}"`);
                await this.sdk.impersonate(this.flags.as, async () => this.runSafe());
            }
            else {
                await this.runSafe();
            }
        }
        catch (error) {
            const stack = error.kuzzleStack || error.stack;
            const errorLink = typeof error.id === "string" && error.id.split(".").length === 3
                ? ` (https://docs.kuzzle.io/core/2/api/errors/error-codes/${error.id.split(".")[0]})`
                : "";
            this.logKo(`Error stack: \n${stack || error.message}\n\nError status: ${error.status}\n\nError id: ${error.id}${errorLink}`);
            if (Array.isArray(error.errors)) {
                for (const e of error.errors) {
                    this.logKo(`${e.document._id} : ${e.reason}`);
                }
            }
            else {
                this.logKo(JSON.stringify(error.reason));
            }
            err = true;
            this.exitCode = 1;
        }
        finally {
            if (kommand.id !== undefined) {
                await Promise.race([
                    this.telemetry.add({
                        action: kommand.id,
                        product: this.config.name,
                        version: this.config.version,
                        tags: { err },
                    }),
                    new Promise((resolve) => setTimeout(resolve, 500)),
                ]);
            }
            this.sdk.disconnect();
            process.exit(this.exitCode);
        }
        return this.exitCode;
    }
    beforeConnect() {
        // will be called before connecting to Kuzzle
    }
    async runSafe() {
        throw new Error("You must implement runSafe() method");
    }
    /**
     * Reads a value from STDIN.
     *
     * @returns {String} Parsed input
     */
    fromStdin() {
        // cucumber mess with stdin so I have to do this trick
        if (process.env.NODE_ENV === "test" || process.stdin.isTTY) {
            return;
        }
        return fs_1.default.readFileSync(0, "utf8");
    }
    fromEditor(defaultContent, options) {
        let content = defaultContent;
        if (typeof content !== "string") {
            content = JSON.stringify(content);
        }
        const editor = new editor_1.Editor(content, options);
        editor.run();
        return this.parseJs(editor.content);
    }
    parseJs(input) {
        if (!input) {
            return {};
        }
        // eslint-disable-next-line no-eval
        return eval(`var o = ${input}; o`);
    }
}
exports.Kommand = Kommand;
Kommand.initSdk = true;
Kommand.disableLog = false;
Kommand.readStdin = false;
Kommand.keepAuth = false;
/**
 * An iteration-order-safe version of lodash.values
 *
 * @param object The object containing the values
 * @param fields The field names to pick in the right order
 * @returns The values in the same order as the fields
 * @see https://lodash.com/docs/4.17.15#values
 */
function pickValues(object, fields) {
    return fields.map((f) => formatValueForCSV((0, get_1.default)(object, f)));
}
exports.pickValues = pickValues;
/**
 * Formats the value for correct CSV output, avoiding to return
 * values that would badly serialize in CSV.
 *
 * @param value The value to format
 * @returns The value or a string telling the value is not scalar
 */
function formatValueForCSV(value) {
    if ((0, isObject_1.default)(value)) {
        return "[NOT_SCALAR]";
    }
    return value;
}
exports.formatValueForCSV = formatValueForCSV;
