"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute = exports.ExecutionError = void 0;
const child_process_1 = require("child_process");
class ExecutionError extends Error {
    constructor(stderr, stdout, code, command) {
        super(stderr);
        this.result = { stdout, stderr, exitCode: code };
        this.command = command.join(" ");
    }
}
exports.ExecutionError = ExecutionError;
/**
 * Executes a command.
 *
 * The given should be given as strings as for the child_process.spawn command.
 * If the last argument is an object, it will be passed to the child_process.spawn command.
 *
 * @example
 * await execute('ls', '-la')
 * await execute('ls', '-la', { cwd: '/home/aschen' })
 * @returns {ProcessExecutor<ExecutionResult>} ProcessExecutor
 */
function execute(...args) {
    let options;
    if (typeof args[args.length - 1] === "object") {
        options = args.splice(args.length - 1)[0];
    }
    const [command, ...commandArgs] = args;
    const process = (0, child_process_1.spawn)(command, commandArgs, options);
    let stdout = "";
    let stderr = "";
    process.stdout.on("data", (data) => (stdout += data.toString()));
    process.stderr.on("data", (data) => (stderr += data.toString()));
    const executor = new Promise((resolve, reject) => {
        process.on("close", (code) => {
            if (code === 0) {
                resolve({ stdout, stderr, exitCode: code });
            }
            else {
                reject(new ExecutionError(stderr, stdout, code, args));
            }
        });
    });
    executor.process = process;
    return executor;
}
exports.execute = execute;
