"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KuzzleSDK = exports.kuzzleFlags = void 0;
const tslib_1 = require("tslib");
const http_1 = (0, tslib_1.__importDefault)(require("http"));
const https_1 = (0, tslib_1.__importDefault)(require("https"));
const command_1 = require("@oclif/command");
const kuzzle_sdk_1 = require("kuzzle-sdk");
const SECOND = 1000;
exports.kuzzleFlags = {
    host: command_1.flags.string({
        description: "Kuzzle server host",
        default: process.env.KUZZLE_HOST || "localhost",
    }),
    port: command_1.flags.string({
        description: "Kuzzle server port",
        default: process.env.KUZZLE_PORT || "7512",
    }),
    ssl: command_1.flags.boolean({
        description: "Use SSL to connect to Kuzzle",
        default: Boolean(process.env.KUZZLE_SSL) || undefined,
    }),
    username: command_1.flags.string({
        description: "Kuzzle username (local strategy)",
        default: process.env.KUZZLE_USERNAME || "anonymous",
    }),
    password: command_1.flags.string({
        description: "Kuzzle user password",
        default: process.env.KUZZLE_PASSWORD || undefined,
    }),
    protocol: command_1.flags.string({
        description: "Kuzzle protocol (http or ws)",
        default: process.env.KUZZLE_PROTOCOL || "ws",
    }),
    as: command_1.flags.string({
        description: "Impersonate a user",
    }),
    "api-key": command_1.flags.string({
        description: "Kuzzle user api-key",
        default: process.env.KUZZLE_API_KEY || undefined,
    }),
};
class KuzzleSDK {
    constructor(options) {
        this.host = options.host;
        this.port = parseInt(options.port, 10);
        this.ssl = options.ssl || this.port === 443;
        this.username = options.username;
        this.password = options.password;
        this.protocol = options.protocol;
        this.apikey = options["api-key"] || options.apiKey;
        this.appVersion = options.appVersion;
        this.appName = options.appName;
        this.ttl = options.ttl || "90s";
        this.keepAuth = options.keepAuth || false;
        // Instantiate a fake SDK in the constructor to please TS
        this.sdk = new kuzzle_sdk_1.Kuzzle(new kuzzle_sdk_1.WebSocket("nowhere"));
    }
    async init(logger) {
        let ProtocolClass;
        // Avoid common mistake
        if (this.protocol === "websocket") {
            this.protocol = "ws";
        }
        if (this.protocol === "ws") {
            ProtocolClass = kuzzle_sdk_1.WebSocket;
        }
        else if (this.protocol === "http") {
            ProtocolClass = kuzzle_sdk_1.Http;
        }
        else {
            throw new TypeError(`Unknown protocol "${this.protocol}"`);
        }
        this.sdk = new kuzzle_sdk_1.Kuzzle(new ProtocolClass(this.host, {
            port: this.port,
            sslConnection: this.ssl,
            pingInterval: 20 * 1000,
        }));
        this.sdk.volatile = {
            client: `${this.appName}@${this.appVersion}`,
        };
        this.sdk.on("networkError", (error) => logger.logKo(error.message));
        logger.logInfo(`Connecting to ${this.protocol}${this.ssl ? "s" : ""}://${this.host}:${this.port} ...`);
        await this.sdk.connect();
        if (this.apikey) {
            this.sdk.jwt = this.apikey;
        }
        else if (this.username !== "anonymous") {
            const credentials = {
                username: this.username,
                password: this.password,
            };
            await this.sdk.auth.login("local", credentials, this.ttl);
            if (this.keepAuth) {
                this.refreshTimer = setInterval(async () => {
                    try {
                        await this.sdk.auth.refreshToken();
                    }
                    catch (error) {
                        logger.logKo(`Cannot refresh token: ${error.message}`);
                    }
                }, 80 * SECOND);
            }
            logger.logInfo(`Loggued as ${this.username}.`);
        }
    }
    /**
     * Impersonates a user.
     * Every action called in the given callback will be impersonated.
     *
     * @param {string} userKuid - User kuid to impersonate
     * @param {Function} callback - Callback that will be impersonated
     * @returns {void}
     */
    async impersonate(userKuid, callback) {
        const currentToken = this.sdk.jwt;
        let apiKey;
        try {
            apiKey = await this.security.createApiKey(userKuid, "Kourou impersonation token", { expiresIn: "2h", refresh: false });
            this.sdk.jwt = apiKey._source.token;
            await callback();
        }
        finally {
            this.sdk.jwt = currentToken;
            if (apiKey === null || apiKey === void 0 ? void 0 : apiKey._id) {
                await this.security.deleteApiKey(userKuid, apiKey._id);
            }
        }
    }
    disconnect() {
        this.sdk.disconnect();
        if (this.refreshTimer) {
            clearInterval(this.refreshTimer);
        }
    }
    query(request) {
        // Convert string to boolean when protocol is WebSocket
        if (this.protocol === "ws") {
            for (const [key, value] of Object.entries(request)) {
                if (value === "true") {
                    request[key] = true;
                }
                else if (value === "false") {
                    request[key] = false;
                }
            }
        }
        return this.sdk.query(request);
    }
    /**
     * Query the Kuzzle API and return a streamed response.
     * @param request The request to send to Kuzzle
     * @returns The response stream
     */
    queryHttpStream(request) {
        // Ensure the protocol is HTTP
        if (this.protocol !== "http") {
            throw new TypeError("HTTP streaming is only available with the HTTP protocol");
        }
        // Construct the URL
        const url = `${this.ssl ? "https" : "http"}://${this.host}:${this.port}/_query`;
        // Construct the request
        const body = JSON.stringify(request);
        const options = {
            method: "POST",
            headers: {
                Accept: "application/json",
                "Accept-Encoding": "identity",
                Authorization: `Bearer ${this.sdk.jwt}`,
                Connection: "keep-alive",
                "Content-Length": Buffer.byteLength(body),
                "Content-Type": "application/json",
            },
        };
        // Send the request
        return new Promise((resolve, reject) => {
            const httpModule = this.ssl ? https_1.default : http_1.default;
            const req = httpModule.request(url, options, (res) => {
                resolve(res);
            });
            req.on("error", reject);
            req.write(body);
            req.end();
        });
    }
    get document() {
        return this.sdk.document;
    }
    get collection() {
        return this.sdk.collection;
    }
    get index() {
        return this.sdk.index;
    }
    get security() {
        return this.sdk.security;
    }
    get auth() {
        return this.sdk.auth;
    }
    get server() {
        return this.sdk.server;
    }
    get realtime() {
        return this.sdk.realtime;
    }
    get ms() {
        return this.sdk.ms;
    }
}
exports.KuzzleSDK = KuzzleSDK;
