"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Elasticsearch = void 0;
const elasticsearch_1 = require("@elastic/elasticsearch");
class Elasticsearch {
    constructor(url, options) {
        this.client = new elasticsearch_1.Client({ node: url });
        this.options = options;
    }
    async listIndices(pattern) {
        const { body } = await this.client.cat.indices({ format: "json" });
        return body
            .map(({ index }) => index)
            .filter((index) => index.match(pattern))
            .filter((index) => !index.startsWith(".")) // ignore system indices
            .sort();
    }
    async getIndex(index) {
        const resp = await this.client.indices.get({ index });
        // Remove unsupported settings properties
        const specifications = resp.body[index];
        delete specifications.settings.index.creation_date;
        delete specifications.settings.index.provided_name;
        delete specifications.settings.index.uuid;
        delete specifications.settings.index.version;
        delete specifications.settings.index.routing;
        delete specifications.settings.index.history;
        return specifications;
    }
    async createIndex(index, specifications) {
        specifications.settings.write = {
            wait_for_active_shards: 1, // Wait for at least one shard to be active before returning
        };
        await this.client.indices.create({ index, body: specifications });
    }
    async readData(index, args) {
        if (!args || !args.scrollId) {
            const { body } = await this.client.search({
                index,
                scroll: this.options.scrollDuration,
                body: {
                    query: {
                        match_all: {},
                    },
                },
                size: this.options.batchSize,
            });
            return {
                documents: body.hits.hits,
                total: body.hits.total.value,
                scrollId: body._scroll_id
            };
        }
        else {
            const { body: { hits }, } = await this.client.scroll({
                scroll_id: args.scrollId,
                scroll: this.options.scrollDuration
            });
            return hits.hits;
        }
    }
    async writeData(index, docs) {
        const bulk = [];
        for (const doc of docs) {
            bulk.push({ index: { _index: index, _id: doc._id } });
            bulk.push(doc._source);
        }
        await this.client.bulk({ body: bulk });
        return bulk.length / 2;
    }
    async clear() {
        await this.client.indices.delete({ index: "_all" });
    }
}
exports.Elasticsearch = Elasticsearch;
