"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.restoreCollectionMappings = exports.restoreCollectionData = void 0;
const tslib_1 = require("tslib");
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const chalk_1 = (0, tslib_1.__importDefault)(require("chalk"));
const ndjson_1 = (0, tslib_1.__importDefault)(require("ndjson"));
function handleError(log, dumpFile, error) {
    if (error.errors) {
        const errorFile = `${dumpFile
            .split(".")
            .slice(0, -1)
            .join(".")}-errors.jsonl`;
        const writeStream = fs_1.default.createWriteStream(errorFile, { flags: "a" });
        for (const partialError of error.errors) {
            writeStream.write(JSON.stringify(partialError) + '\n');
        }
        writeStream.end();
        log(chalk_1.default.red(`[X] Error importing ${dumpFile}. See errors in ${errorFile}`));
    }
    else {
        log(chalk_1.default.red(error.message));
        throw error;
    }
}
async function restoreCollectionData(sdk, log, batchSize, dumpFile, index, collection, migrateDocument) {
    const mWriteRequest = {
        controller: "bulk",
        action: "mWrite",
        index: "",
        collection: "",
        strict: true,
        body: {
            documents: [{}],
        },
    };
    let total = 0;
    await new Promise((resolve, reject) => {
        let headerSkipped = false;
        let documents = [];
        const readStream = fs_1.default
            .createReadStream(dumpFile)
            .pipe(ndjson_1.default.parse())
            .on("data", (obj) => {
            if (headerSkipped) {
                const document = migrateDocument ? migrateDocument(obj) : obj;
                if (!document._id) {
                    throw new Error(`Document does not have an "_id" property: ${JSON.stringify(document)}`);
                }
                if (!document.body) {
                    throw new Error(`Document does not have an "body" property: ${JSON.stringify(document)}`);
                }
                documents.push(document);
                if (documents.length === batchSize) {
                    mWriteRequest.body.documents = documents;
                    documents = [];
                    readStream.pause();
                    sdk
                        .query(mWriteRequest)
                        .then(() => {
                        total += mWriteRequest.body.documents.length;
                        process.stdout.write(`  ${total} documents imported`);
                        process.stdout.write("\r");
                        readStream.resume();
                    })
                        .catch((error) => {
                        try {
                            handleError(log, dumpFile, error);
                            readStream.resume();
                        }
                        catch (e) {
                            readStream.end();
                            reject(e);
                        }
                    });
                }
            }
            else {
                if (obj.type !== "collection") {
                    throw new Error("Dump file does not contains collection data");
                }
                headerSkipped = true;
                mWriteRequest.index = index || obj.index;
                mWriteRequest.collection = collection || obj.collection;
            }
        })
            .on("end", () => {
            if (documents.length > 0) {
                mWriteRequest.body.documents = documents;
                sdk
                    .query(mWriteRequest)
                    .then((response) => {
                    total += response.result.successes.length;
                    process.stdout.write(`  ${total} documents imported`);
                    process.stdout.write("\r");
                    resolve(undefined);
                })
                    .catch((error) => {
                    try {
                        handleError(log, dumpFile, error);
                        reject(error);
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            }
            else {
                resolve(undefined);
            }
        });
    });
    return {
        total,
        collection: mWriteRequest.collection,
        index: mWriteRequest.index,
    };
}
exports.restoreCollectionData = restoreCollectionData;
/**
 * Imports mappings from a collection mappings dump
 * Expected format:
 * {
 *   type: 'mappings',
 *   content: {
 *     index: {
 *       collection: {
 *         // mappings
 *       }
 *     }
 *   }
 * }
 *
 * @param {Kuzzle} sdk - Kuzzle SDK instance
 * @param {String} dump - Dump object (type, content)
 * @param {String} index - Override index name
 * @param {String} collection - Override collection name
 * @returns {void}
 */
async function restoreCollectionMappings(sdk, dump, index, collection) {
    if (dump.type !== "mappings") {
        throw new Error("Dump file does not contain mappings definition");
    }
    const srcIndex = Object.keys(dump.content)[0];
    const srcCollection = Object.keys(dump.content[srcIndex])[0];
    const dstIndex = index || srcIndex;
    const dstCollection = collection || srcCollection;
    if (!(await (sdk === null || sdk === void 0 ? void 0 : sdk.index.exists(dstIndex)))) {
        await (sdk === null || sdk === void 0 ? void 0 : sdk.index.create(dstIndex));
    }
    await (sdk === null || sdk === void 0 ? void 0 : sdk.collection.create(dstIndex, dstCollection, dump.content[srcIndex][srcCollection]));
    return {
        index: dstIndex,
        collection: dstCollection,
    };
}
exports.restoreCollectionMappings = restoreCollectionMappings;
